#!/usr/bin/python3
from __future__ import annotations
from typing import List
import requests
from dataclasses import dataclass
from bs4 import BeautifulSoup


@dataclass
class Course:
    id: str
    name: str
    group: str


@dataclass
class ScheduleCourse:
    course: Course
    weekDay: str
    period: str
    theater: str


class CourseCrawler:

    def __init__(self) -> CourseCrawler:
        self.session = requests.Session()

    SCHEDULE_URL = "http://112.137.129.115/tkb/listbylist.php"
    COURSE_URL = "http://112.137.129.87/qldt"
    DEFAULT_PAYLOAD = {
        "SinhvienLmh[masvTitle]": "",
        "SinhvienLmh[hotenTitle]": "",
        "SinhvienLmh[ngaysinhTitle]": "",
        "SinhvienLmh[lopkhoahocTitle]": "",
        "SinhvienLmh[tenlopmonhocTitle]": "",
        "SinhvienLmh[tenmonhocTitle]": "",
        "SinhvienLmh[nhom]": "",
        "SinhvienLmh[sotinchiTitle]": "",
        "SinhvienLmh[ghichu]": "",
        "SinhvienLmh[term_id]": "",
        "SinhvienLmh_page": "1",
        "ajax": "sinhvien-lmh-grid",
        "pageSize": "5000"
    }

    def create_params(self, student_id: str, term_id: str):
        params = CourseCrawler.DEFAULT_PAYLOAD.copy()
        params["SinhvienLmh[masvTitle]"] = student_id
        params["SinhvienLmh[term_id]"] = term_id

        return params

    def auto_detect_term_id(self) -> str:
        """Auto detect term id code if not passed"""
        res = self.session.get(CourseCrawler.COURSE_URL)
        soup = BeautifulSoup(res.text, "lxml")

        term_ids = [tag["value"] for tag in soup.select("#SinhvienLmh_term_id>option")]
        # Pick the lastest term
        return max(term_ids)

    def fetch_courses(self, student_id: str, term_id: str = None) -> List[Course]:
        """Make API requests to get coursess of student"""

        if term_id is None:
            term_id = self.auto_detect_term_id()

        payload = self.create_params(student_id, term_id)
        res = self.session.get(CourseCrawler.COURSE_URL, params=payload)

        soup = BeautifulSoup(res.text, "lxml")
        table = soup.select_one("div#sinhvien-lmh-grid")

        courses = []
        for row in table.select("tbody>tr"):
            columns = list(map(lambda tag: str(tag.get_text()), row.findChildren(name="td")))
            _id, name, group = columns[5:8]  # interesting column

            courses.append(Course(_id, name, group))

        return courses

    def fetch_schedule(self, courses: List[Course]) -> List[ScheduleCourse]:

        res = self.session.get(CourseCrawler.SCHEDULE_URL)

        soup = BeautifulSoup(res.text, "lxml")
        table = soup.select_one("table:nth-child(4)")

        schedule_courses = []
        course_ids = {course.id: course for course in courses}
        for row in table.select("tr"):
            columns = list(map(lambda tag: tag.get_text(), row.findChildren(name="td")))

            # Not a valid column
            if len(columns) < 12:
                continue

            _id = columns[4]
            name = columns[2]
            group = columns[11]
            weekDay = columns[8]
            period = columns[9]
            theater = columns[10]

            if _id not in course_ids.keys():
                continue

            if group != "CL" and group != course_ids[_id].group:
                continue

            course = Course(_id, name, group)
            schedule_course = ScheduleCourse(course, weekDay, period, theater)

            schedule_courses.append(schedule_course)

        return schedule_courses


if __name__ == "__main__":
    crawler = CourseCrawler()
    print(crawler.fetch_courses("21020782"))



from __future__ import print_function, annotations
from typing import List

from dataclasses import dataclass
from datetime import datetime
from dateutil import tz
from google.oauth2 import service_account
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

# If modifying these scopes, delete the file token.json.
SCOPES = ["https://www.googleapis.com/auth/calendar"]

KEY_FILE_PATH = "secret/autoschedule.json"

# Security risk, i admit.
CLIENT_INFO = {
    "type": "service_account",
    "project_id": "autoschedule-377806",
    "private_key_id": "6cb2e5f42e653c3caa4a54702a947bc73b1ca509",
    "private_key": "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQDQoF7EFnb+QOv4\nzEK1/U+EIMhmXvW+z8nuTNVKDFZhbBMzx1QpcRlTZPA0qsbXIBcDbLx6pgDGTq6X\nXnLm9L/Ocx7uTGkV5O/0EIBUGos0ndovKWdqXmZZuU5JVAmco2/OYy5YPpZ4OCXG\nxa5Oc9HlFxJbAYPgs7h77HuvpHJ5vNoP9+xA1gw2zm1dENgQx2JKHIRxv83D55zQ\nYbf8nZAtMvBmDhvB0vI+0CVQNpCSXCS81bGZw3XHgcM6IUgj+1AiSigiNdFVSdBT\nMM6SQVVed09LnktmYhgBR7mKm9U5mrzEYFf3KzuQUo2ZiJ5pHblJylgFcfnKHWAH\nkuiKWdgBAgMBAAECggEACDwWcvQ+OrMfrsT/DXb/ipV9Z93BUUKyk6yR+qqIXTsn\neicY1Zx4aHC5yiGr475yD+rRG+aWbckOL7FcPD5tzBewz9uunXRZQ/VmyGsQhwc9\n2V3MvOqMD9yTbU7Paq3Uz06YKoadLmP4PWN8rKfi8vWtuED2yKD1OR5E8QDh4Jei\nz9EYwtiYV1jDdvSLnnmxkNni811V41QVK0zDX0/N56v63emRfmdBT8Cd3idbCcga\njUMaDtgHnJLMiMf8bbNXsWMJTgsg0YRW67TA2wHk/uHlTtsOteEilxgeSkJewx0j\nUbTNeMqT6M6wlcEVrus3OgOcIzez3dT21w1gFHUWlQKBgQDxawLzNW0gBPBm2duO\nqpXVFkstitz1d2w/wM6fjQPkFKO+4fDlRuYy+tSEh036vVv56aw1aCBNR2oAYUs4\nDupTsVgTtLbRmztK9UdhYsuzjO3NqDWuB1eW2XuzAtOxgRMrGmgATnl28CI7uYks\nVH9SKhuUB1FeIPf2mG8uNNOz9wKBgQDdOk+Axhv7js94MfehNqtTiKypZPkWLx6j\nwTYchaI05aQA3SEF797JGSFDQFEt8Wg1KWubHJ8XdHk2Fz+eLflJM1PJfJOdbase\nV3CCgRDjHs+rpnw+T02b7w6QEeoHIGNOOzxykCxwr0w+HEERp2sUmgR4hM4i5k3g\nFJpTp1vlxwKBgGxbEoTkuuj/LoOLTs9W408z5WQ4inomMtDekh7spQ1u8hvPv1a5\nHsNM76vYKvjamvitl9yUssLcxcgL4z3y+9u6ooSdvNyRZuh69KHSPQmGvIls8UL+\nCqMrVCkBoNBv1ZGoFXvlvQDUQRk7akUaW9ceDSjArBWqiIEG5AgCL+77AoGAUNX8\ntPYuxWWyg53meR7rzXM0fBUsOyHulTGFXzipQ6Dho3splOzIQD0RfWxj/WCnjj1b\nV2tG6qAhplUqZtcvrsK1i/scSTsIeubCcr1MeWEJyxPjdDUwC5l3fcc992qrPqvA\nc7AgvPAg8NUMJbJCG14H4i3M0M3MiOOeXJDbsIMCgYEA6cnS2xY2wBuwZUuG5hQ+\nY5e+JSlSs46LvnQBbMyTqjbPkSolxWT+ft4Ii9BtM0ScWZ1IGXdNqljj7h8bFT1c\nzRtwBTHfBkx+DUM/i8yNBHipn+S1TonNO0LBADNyMJzbLs/2fUsCvcz7AP93t7Yr\nw1qqBUJV0VLXG5qrvRELBZE=\n-----END PRIVATE KEY-----\n",
    "client_email": "schedulebot@autoschedule-377806.iam.gserviceaccount.com",
    "client_id": "104157667300773377339",
    "auth_uri": "https://accounts.google.com/o/oauth2/auth",
    "token_uri": "https://oauth2.googleapis.com/token",
    "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
    "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/schedulebot%40autoschedule-377806.iam.gserviceaccount.com"
}


@dataclass
class Calendar:
    id: str
    name: str


@dataclass
class Event:
    title: str
    start_time: datetime
    end_time: datetime


class GoogleCalendarService:

    def __init__(self):
        self.service = None

        self.authenticate()

    def authenticate(self):
        credentials = service_account.Credentials.from_service_account_info(
            CLIENT_INFO, scopes=SCOPES
        )

        self.service = build("calendar", "v3", credentials=credentials)

    def get_calendars(self) -> List[Calendar]:
        try:
            calendar_list = self.service.calendarList().list().execute()

            calendars = []
            for calendar_data in calendar_list["items"]:
                calendar = Calendar(calendar_data["id"], calendar_data["summary"])
                calendars.append(calendar)

            return calendars
        except HttpError as error:
            print("An error occurred: %s" % error)

        return []

    def share_calendar_with_user(self, calendar_id: str, user_email: str):
        rule = {
            "scope": {
                "type": "user",
                "value": user_email
            },
            "role": "writer"
        }
        self.service.acl().insert(calendarId=calendar_id, body=rule).execute()

    def create_calendar(self, summary: str) -> Calendar:
        body = {
            "summary": summary
        }

        google_created_calendar = self.service.calendars().insert(body=body).execute()
        calendar = Calendar(google_created_calendar["id"], google_created_calendar["summary"])

        return calendar

    def delete_calendar(self, calendar_id: str):
        try:
            self.service.calendars().delete(calendarId=calendar_id).execute()
            print("Calendar with ID {} has been deleted".format(calendar_id))
        except HttpError as error:
            print('An error occurred: %s' % error)

    def batch_multiple_events(self, events: List[Event], calendar: Calendar = None):

        google_calendar_events = [self._convert_to_google_calendar_event(event)
                                  for event in events]

        # Batch insert events
        try:
            batch = self.service.new_batch_http_request()
            calendar_id = calendar.id if calendar is not None else "primary"

            for e in google_calendar_events:
                batch.add(self.service.events().insert(calendarId=calendar_id, body=e))

            batch.execute()
        except HttpError as error:
            print('An error occurred: %s' % error)

        pass

    def _convert_to_google_calendar_event(self, event: Event):
        start_time = event.start_time.replace(tzinfo=tz.tzlocal())
        end_time = event.end_time.replace(tzinfo=tz.tzlocal())

        event_body = {
            "summary": event.title,
            "start": {
                "dateTime": start_time.isoformat(),
                "timeZone": start_time.tzname(),
            },
            "end": {
                "dateTime": end_time.isoformat(),
                "timeZone": end_time.tzname(),
            },
            "reminders": {
                "useDefault": False,
                "overrides": [
                    {"method": "popup", "minutes": 30},
                ],
            },
        }

        return event_body

    def create_event(self, event: Event, calendar: Calendar = None):
        self.batch_multiple_events([event], calendar)



import click
import re
from . import schedule
from .schedule import SchedulerService


def validate_student_id(ctx, param, value):
    """Callback function that validates student ID."""
    if not re.match(r"^\d{8}$", value):
        raise click.BadParameter("Invalid student ID. Must have 8-digits (e.g 21020782).")
    return value


def validate_email(ctx, param, value):
    """Callback function that validates an email address."""
    if not re.match(r"[^@]+@[^@]+\.[^@]+", value):
        raise click.BadParameter("Invalid email address.")
    return value


@click.command()
@click.option("--student-id", callback=validate_student_id, required=True, help="The id of student.")
@click.option("--user-mail", callback=validate_email, required=True, help="The google account to create event.")
@click.option("--calendar-name", default=schedule.DEFAULT_CALENDAR, help="The calendar name. Default `Schedule`.")
def main(student_id: str, user_mail: str, calendar_name: str):
    service = SchedulerService()
    service.schedule_courses(student_id, user_mail, calendar_name)


if __name__ == '__main__':
    main()



from typing import List
from .crawler import ScheduleCourse, CourseCrawler
from .calendars import Event, GoogleCalendarService

import webbrowser
import halo
from dateutil import rrule
from datetime import time, datetime, timedelta

SEMESTER_WEEKS = 16

DEFAULT_CALENDAR = "Schedule"

LESSON_TIME = {
    "1": time(7, 0, 0),
    "2": time(8, 0, 0),
    "3": time(9, 0, 0),
    "4": time(10, 0, 0),
    "5": time(11, 0, 0),
    "6": time(12, 0, 0),
    "7": time(13, 0, 0),
    "8": time(14, 0, 0),
    "9": time(15, 0, 0),
    "10": time(16, 0, 0),
    "11": time(17, 0, 0),
    "12": time(18, 0, 0),
    "13": time(18, 0, 0),
    "14": time(19, 0, 0),
    "15": time(20, 0, 0),
}


class SchedulerService:
    def __init__(self) -> None:
        self.crawler = CourseCrawler()
        self.calendar_service = GoogleCalendarService()

    def fetch_schedule_from_server(self, student_id: str, term_id: str = None):
        student_courses = self.crawler.fetch_courses(student_id, term_id)
        scheduled_courses = self.crawler.fetch_schedule(student_courses)

        return scheduled_courses

    def schedule_courses(self, student_id: str, user_mail: str, calendar_name: str = DEFAULT_CALENDAR):
        """
        Schedule student courses in google calenlar and share it to user_mail.
        This also open browser to that URL
        student_id: The UET student id. E.g 21020782
        user_mail: An normal gmail abc@gmail.com.
        They are stored in info dictionary. e.g info["student_id"]
        """

        spinner = halo.Halo(text="Fetch schedule from school API", spinner="dots")
        spinner.start()
        scheduled_courses = self.fetch_schedule_from_server(student_id)
        spinner.stop_and_persist("✔", f"Got {len(scheduled_courses)} courses.")

        spinner.text = "Create the calendar."
        spinner.start()
        calendar = self.calendar_service.create_calendar(calendar_name)
        self.calendar_service.share_calendar_with_user(calendar.id, user_mail)
        all_events = []
        for course in scheduled_courses:
            course_events = self.create_course_events(course)
            all_events.extend(course_events)
        spinner.stop_and_persist("✔", f"Succeeded. {calendar_name} is created.")

        spinner.text = "Create events on Google Calendar. This may take a few minutes."
        spinner.start()

        self.calendar_service.batch_multiple_events(all_events, calendar)

        spinner.stop_and_persist("✔", "Done, opening the browser.")
        invite_link = f"https://calendar.google.com/calendar/u/0?cid={calendar.id}&invite={user_mail}"
        # Open invite link in user browser
        webbrowser.open(invite_link)

    def convert_period_to_time(self, period: str):
        """
        The period is in the format number-number. E.g 1-2, where number is from 1 to 14
        Convert in to actual lesson time (7:00, 9:00)
        """
        start, end = period.split("-")

        return LESSON_TIME[start], LESSON_TIME[str(int(end) + 1)]

    def create_course_events(self, course: ScheduleCourse, repeat=SEMESTER_WEEKS) -> List[Event]:
        start, end = self.convert_period_to_time(course.period)

        week_rule = rrule.weekday(int(course.weekDay) - 2)
        start_rule = rrule.rrule(
            rrule.WEEKLY,
            dtstart=datetime.now() - timedelta(weeks=1),
            byweekday=week_rule,
            count=repeat,
            byhour=start.hour,
            byminute=start.minute,
            bysecond=start.second
        )
        end_rule = rrule.rrule(
            rrule.WEEKLY,
            dtstart=datetime.now() - timedelta(weeks=1),
            byweekday=week_rule,
            count=repeat,
            byhour=end.hour,
            byminute=end.minute,
            bysecond=end.second
        )

        events = []
        for start_time, end_time in zip(start_rule, end_rule):
            title = f"{course.course.name} {course.course.group} - {course.theater}"
            event = Event(title, start_time, end_time)
            events.append(event)

        return events


if __name__ == "__main__":
    student_id = "21020782"
    term_id = "036"

    scheduler = SchedulerService()

    scheduler.schedule_courses(student_id, term_id)



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='UETSchedule',
    version='0.1.5',
    description='A script to auto schedule UET courses',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ppvan/autotkb',
    author='ppvan',
    author_email='phuclaplace@gmail.com',
    license='GPL-3.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords=['uet', 'schedule'],
    python_requires='>=3.7',
    packages=find_packages(),
    install_requires=open("requirements.txt").read().splitlines(),
    entry_points={
        'console_scripts': [
            'schedule = schedule.cli:main',
        ],
    },
)
